// **********************************************************************
//
// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

// Ice version 3.2.1

// Freeze types in this file:
// name="IceGrid::IdentityObjectInfoDict", key="Ice::Identity", value="IceGrid::ObjectInfo"

#include <Ice/BasicStream.h>
#include <IceGrid/IdentityObjectInfoDict.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 302
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 1
#       error Ice patch level mismatch!
#   endif
#endif

void
IceGrid::IdentityObjectInfoDictKeyCodec::write(const ::Ice::Identity& v, Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    v.__write(&stream);
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceGrid::IdentityObjectInfoDictKeyCodec::read(::Ice::Identity& v, const Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    v.__read(&stream);
}

static const ::std::string __IceGrid__IdentityObjectInfoDictKeyCodec_typeId = "::Ice::Identity";

const ::std::string&
IceGrid::IdentityObjectInfoDictKeyCodec::typeId()
{
    return __IceGrid__IdentityObjectInfoDictKeyCodec_typeId;
}

void
IceGrid::IdentityObjectInfoDictValueCodec::write(const ::IceGrid::ObjectInfo& v, Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.startWriteEncaps();
    v.__write(&stream);
    stream.endWriteEncaps();
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceGrid::IdentityObjectInfoDictValueCodec::read(::IceGrid::ObjectInfo& v, const Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    stream.startReadEncaps();
    v.__read(&stream);
    stream.endReadEncaps();
}

static const ::std::string __IceGrid__IdentityObjectInfoDictValueCodec_typeId = "::IceGrid::ObjectInfo";

const ::std::string&
IceGrid::IdentityObjectInfoDictValueCodec::typeId()
{
    return __IceGrid__IdentityObjectInfoDictValueCodec_typeId;
}

IceGrid::IdentityObjectInfoDict::TypeIndex::TypeIndex(const std::string& __name, const Freeze::IceEncodingCompare& __compare)
    : Freeze::MapIndex< ::std::string, TypeIndex, Freeze::IceEncodingCompare >(__name, __compare)
{
}

void
IceGrid::IdentityObjectInfoDict::TypeIndex::marshalKey(const Freeze::Value& __v, Freeze::Key& __k) const
{
    ::IceGrid::ObjectInfo __x;
    IceGrid::IdentityObjectInfoDictValueCodec::read(__x, __v, _communicator);
    write(__x.type, __k, _communicator);
}

void
IceGrid::IdentityObjectInfoDict::TypeIndex::write(const ::std::string& __index, Freeze::Key& __bytes, const Ice::CommunicatorPtr& __communicator)
{
    IceInternal::InstancePtr __instance = IceInternal::getInstance(__communicator);
    IceInternal::BasicStream __stream(__instance.get());
    __stream.write(__index);
    ::std::vector<Ice::Byte>(__stream.b.begin(), __stream.b.end()).swap(__bytes);
}

void
IceGrid::IdentityObjectInfoDict::TypeIndex::read(::std::string& __index, const Freeze::Key& __bytes, const Ice::CommunicatorPtr& __communicator)
{
    IceInternal::InstancePtr __instance = IceInternal::getInstance(__communicator);
    IceInternal::BasicStream __stream(__instance.get());
    __stream.b.resize(__bytes.size());
    ::memcpy(&__stream.b[0], &__bytes[0], __bytes.size());
    __stream.i = __stream.b.begin();
    __stream.read(__index);
}

IceGrid::IdentityObjectInfoDict::IdentityObjectInfoDict(const Freeze::ConnectionPtr& __connection, const std::string& __dbName ,bool __createDb, const Freeze::IceEncodingCompare& __compare)
    : Freeze::Map< ::Ice::Identity, ::IceGrid::ObjectInfo, IdentityObjectInfoDictKeyCodec, IdentityObjectInfoDictValueCodec, Freeze::IceEncodingCompare >(__connection->getCommunicator())
{
    Freeze::KeyCompareBasePtr __keyCompare = new Freeze::KeyCompare< ::Ice::Identity, IdentityObjectInfoDictKeyCodec, Freeze::IceEncodingCompare >(__compare, _communicator);
    std::vector<Freeze::MapIndexBasePtr> __indices;
    __indices.push_back(new TypeIndex("type"));
    _helper.reset(Freeze::MapHelper::create(__connection, __dbName, IceGrid::IdentityObjectInfoDictKeyCodec::typeId(), IceGrid::IdentityObjectInfoDictValueCodec::typeId(), __keyCompare, __indices, __createDb));
}

IceGrid::IdentityObjectInfoDict::iterator
IceGrid::IdentityObjectInfoDict::findByType(const ::std::string& __index, bool __onlyDups)
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return iterator(_helper->index("type")->untypedFind(__bytes, false, __onlyDups), _communicator);
}

IceGrid::IdentityObjectInfoDict::const_iterator
IceGrid::IdentityObjectInfoDict::findByType(const ::std::string& __index, bool __onlyDups) const
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return const_iterator(_helper->index("type")->untypedFind(__bytes, true, __onlyDups), _communicator);
}

IceGrid::IdentityObjectInfoDict::iterator
IceGrid::IdentityObjectInfoDict::lowerBoundForType(const ::std::string& __index)
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return iterator(_helper->index("type")->untypedLowerBound(__bytes, false), _communicator);
}

IceGrid::IdentityObjectInfoDict::const_iterator
IceGrid::IdentityObjectInfoDict::lowerBoundForType(const ::std::string& __index) const
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return const_iterator(_helper->index("type")->untypedLowerBound(__bytes, true), _communicator);
}

IceGrid::IdentityObjectInfoDict::iterator
IceGrid::IdentityObjectInfoDict::upperBoundForType(const ::std::string& __index)
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return iterator(_helper->index("type")->untypedUpperBound(__bytes, false), _communicator);
}

IceGrid::IdentityObjectInfoDict::const_iterator
IceGrid::IdentityObjectInfoDict::upperBoundForType(const ::std::string& __index) const
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return const_iterator(_helper->index("type")->untypedUpperBound(__bytes, true), _communicator);
}

std::pair<IceGrid::IdentityObjectInfoDict::iterator, IceGrid::IdentityObjectInfoDict::iterator>
IceGrid::IdentityObjectInfoDict::equalRangeForType(const ::std::string& __index)
{
    return std::make_pair(lowerBoundForType(__index), upperBoundForType(__index));
}

std::pair<IceGrid::IdentityObjectInfoDict::const_iterator, IceGrid::IdentityObjectInfoDict::const_iterator>
IceGrid::IdentityObjectInfoDict::equalRangeForType(const ::std::string& __index) const
{
    return std::make_pair(lowerBoundForType(__index), upperBoundForType(__index));
}

int
IceGrid::IdentityObjectInfoDict::typeCount(const ::std::string& __index) const
{
    Freeze::Key __bytes;
    TypeIndex::write(__index, __bytes, _communicator);
    return _helper->index("type")->untypedCount(__bytes);
}
