// **********************************************************************
//
// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

// Ice version 3.2.1

// Freeze types in this file:
// name="IceGrid::StringAdapterInfoDict", key="string", value="IceGrid::AdapterInfo"

#ifndef __StringAdapterInfoDict_h__
#define __StringAdapterInfoDict_h__

#include <Freeze/Map.h>
#include <IceGrid/Admin.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 302
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 1
#       error Ice patch level mismatch!
#   endif
#endif

namespace IceGrid
{

class StringAdapterInfoDictKeyCodec
{
public:

    static void write(const ::std::string&, Freeze::Key&, const ::Ice::CommunicatorPtr&);
    static void read(::std::string&, const Freeze::Key&, const ::Ice::CommunicatorPtr&);
    static const std::string& typeId();
};

class StringAdapterInfoDictValueCodec
{
public:

    static void write(const ::IceGrid::AdapterInfo&, Freeze::Value&, const ::Ice::CommunicatorPtr&);
    static void read(::IceGrid::AdapterInfo&, const Freeze::Value&, const ::Ice::CommunicatorPtr&);
    static const std::string& typeId();
};

class StringAdapterInfoDict : public Freeze::Map< ::std::string, ::IceGrid::AdapterInfo, StringAdapterInfoDictKeyCodec, StringAdapterInfoDictValueCodec, Freeze::IceEncodingCompare >
{
public:


    class ReplicaGroupIdIndex : public Freeze::MapIndex< ::std::string, ReplicaGroupIdIndex, Freeze::IceEncodingCompare >
    {
    public:

        ReplicaGroupIdIndex(const std::string&, const Freeze::IceEncodingCompare& = Freeze::IceEncodingCompare());

        static void write(const ::std::string&, Freeze::Key&, const Ice::CommunicatorPtr&);
        static void read(::std::string&, const Freeze::Key&, const ::Ice::CommunicatorPtr&);

    protected:

        virtual void marshalKey(const Freeze::Value&, Freeze::Key&) const;
    };

    StringAdapterInfoDict(const Freeze::ConnectionPtr&, const std::string&, bool = true, const Freeze::IceEncodingCompare& = Freeze::IceEncodingCompare());

    template <class _InputIterator>
    StringAdapterInfoDict(const Freeze::ConnectionPtr& __connection, const std::string& __dbName, bool __createDb, _InputIterator __first, _InputIterator __last, const Freeze::IceEncodingCompare& __compare = Freeze::IceEncodingCompare())
        : Freeze::Map< ::std::string, ::IceGrid::AdapterInfo, StringAdapterInfoDictKeyCodec, StringAdapterInfoDictValueCodec, Freeze::IceEncodingCompare >(__connection->getCommunicator())
    {
        Freeze::KeyCompareBasePtr __keyCompare = new Freeze::KeyCompare< ::std::string, StringAdapterInfoDictKeyCodec, Freeze::IceEncodingCompare >(__compare, this->_communicator);
        std::vector<Freeze::MapIndexBasePtr> __indices;
        __indices.push_back(new ReplicaGroupIdIndex("replicaGroupId"));
        this->_helper.reset(Freeze::MapHelper::create(__connection, __dbName, StringAdapterInfoDictKeyCodec::typeId(), StringAdapterInfoDictValueCodec::typeId(), __keyCompare, __indices, __createDb));
        while(__first != __last)
        {
            put(*__first);
            ++__first;
        }
    }

    iterator findByReplicaGroupId(const ::std::string&, bool = true);
    const_iterator findByReplicaGroupId(const ::std::string&, bool = true) const;
    iterator lowerBoundForReplicaGroupId(const ::std::string&);
    const_iterator lowerBoundForReplicaGroupId(const ::std::string&) const;
    iterator upperBoundForReplicaGroupId(const ::std::string&);
    const_iterator upperBoundForReplicaGroupId(const ::std::string&) const;
    std::pair<iterator, iterator> equalRangeForReplicaGroupId(const ::std::string&);
    std::pair<const_iterator, const_iterator> equalRangeForReplicaGroupId(const ::std::string&) const;
    int replicaGroupIdCount(const ::std::string&) const;
};

}

#endif
