// **********************************************************************
//
// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

// Ice version 3.2.1

// Freeze types in this file:
// name="IceStorm::PersistentTopicMap", key="Ice::Identity", value="IceStorm::LinkRecordSeq"

#include <Ice/BasicStream.h>
#include <IceStorm/PersistentTopicMap.h>

#ifndef ICE_IGNORE_VERSION
#   if ICE_INT_VERSION / 100 != 302
#       error Ice version mismatch!
#   endif
#   if ICE_INT_VERSION % 100 > 50
#       error Beta header file detected
#   endif
#   if ICE_INT_VERSION % 100 < 1
#       error Ice patch level mismatch!
#   endif
#endif

void
IceStorm::PersistentTopicMapKeyCodec::write(const ::Ice::Identity& v, Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    v.__write(&stream);
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceStorm::PersistentTopicMapKeyCodec::read(::Ice::Identity& v, const Freeze::Key& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    v.__read(&stream);
}

static const ::std::string __IceStorm__PersistentTopicMapKeyCodec_typeId = "::Ice::Identity";

const ::std::string&
IceStorm::PersistentTopicMapKeyCodec::typeId()
{
    return __IceStorm__PersistentTopicMapKeyCodec_typeId;
}

void
IceStorm::PersistentTopicMapValueCodec::write(const ::IceStorm::LinkRecordSeq& v, Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.startWriteEncaps();
    if(v.size() == 0)
    {
        stream.writeSize(0);
    }
    else
    {
        ::IceStorm::__write(&stream, &v[0], &v[0] + v.size(), ::IceStorm::__U__LinkRecordSeq());
    }
    stream.endWriteEncaps();
    ::std::vector<Ice::Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
}

void
IceStorm::PersistentTopicMapValueCodec::read(::IceStorm::LinkRecordSeq& v, const Freeze::Value& bytes, const ::Ice::CommunicatorPtr& communicator)
{
    IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
    IceInternal::BasicStream stream(instance.get());
    stream.b.resize(bytes.size());
    ::memcpy(&stream.b[0], &bytes[0], bytes.size());
    stream.i = stream.b.begin();
    stream.startReadEncaps();
    ::IceStorm::__read(&stream, v, ::IceStorm::__U__LinkRecordSeq());
    stream.endReadEncaps();
}

static const ::std::string __IceStorm__PersistentTopicMapValueCodec_typeId = "::IceStorm::LinkRecordSeq";

const ::std::string&
IceStorm::PersistentTopicMapValueCodec::typeId()
{
    return __IceStorm__PersistentTopicMapValueCodec_typeId;
}
